﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace Pshais.Indicators
{
    public class ZLMA_v2_double : IIndicatorScript
    {
        int length1 = 20, length2 = 40;
        int lag1 { get { return (length1 - 1) / 2; } }
        int lag2 { get { return (length2 - 1) / 2; } }
        Core.MAType maType1 = Core.MAType.Sma;
        Core.MAType maType2 = Core.MAType.Sma;

        int deviation = 0;

        double value1, value2;

        #region IIndicatorScript implementation
        public void Init()
        {

        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                int position = instrument.Close.Length - (1 + context.Offset + deviation);
                var close = instrument.Close;
                var price = close[position];

                var zlema1 = Zlema(close.Skip(position - TicksBack1).Take(TicksBack1).ToArray(), length1, maType1);
                var zlema2 = Zlema(close.Skip(position - TicksBack2).Take(TicksBack2).ToArray(), length2, maType2);

                if (zlema1 >= 0
                    && zlema2 >= 0)
                {
                    value1 = zlema1;
                    value2 = zlema2;

                    if (value1 > value2)
                        return IndicatorResult.BuyLong;

                    if (value1 < value2)
                        return IndicatorResult.SellShort;
                }
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

        double Zlema(double[] price, int length, Core.MAType maType)
        {
            /*
            src=close
            ema1=ema(src, length)
            ema2=ema(ema1, length)
            d=ema1-ema2
            zlema=ema1+d
            */
            try
            {
                int beg1, len1;
                var output1 = new double[price.Length];
                var ret = Core.MovingAverage(1, price.Length - 1, price, length, maType, out beg1, out len1, output1);

                int beg2, len2;
                var output2 = new double[output1.Length];
                var ret2 = Core.MovingAverage(beg1, len1 - 1, output1, length, maType, out beg2, out len2, output2);

                /*for (int i = beg1; i < output1.Length; i++)
                {
                    var i2 = i - beg1 + beg2;
                    var ma1 = output1[i];
                    var ma2 = output2[i2]
                }*/

                var ma1 = output1[len1 - 1];
                var ma2 = output2[len2 - 1];
                var d = ma1 - ma2;
                var zlema = ma1 + d;

                return zlema;
            }
            catch
            {
                return -1;
            }
            
        }

        public List<ScriptParameter> GetParameters()
        {
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("Length 1", ScriptParameterType.Integer, length1.ToString()),
                new ScriptParameter("MA Type 1", ScriptParameterType.String, maType1.ToString()),
                new ScriptParameter("Length 2", ScriptParameterType.Integer, length2.ToString()),
                new ScriptParameter("MA Type 2", ScriptParameterType.String, maType2.ToString()),
                new ScriptParameter("Deviation", ScriptParameterType.Integer, deviation.ToString())
            };

            return rtn;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            length1 = ParseInt(parameters["Length 1"], length1);
            maType1 = ParseMAType(parameters["MA Type 1"]);
            length2 = ParseInt(parameters["Length 2"], length2);
            maType2 = ParseMAType(parameters["MA Type 2"]);
            deviation = ParseInt(parameters["Deviation"], deviation);
        }

        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "ZLMA Fast", HexLineColor = "#FF55FF" },
                new ChartRecord() { ChartIndex = 0, Name = "ZLMA Slow", HexLineColor = "#4488FF" }
            };

            return chartLines;
        }

        public List<decimal> GetChartData()
        {
            var chartData = new List<decimal>
            {
                Convert.ToDecimal(value1),
                Convert.ToDecimal(value2)
            };

            return chartData;
        }

        public string Name
        {
            get
            {
                return "pshai's ZLMA v1 (double)";
            }
        }

        public int TicksBack
        {
            get
            {
                var lb1 = Core.MovingAverageLookback(length1, maType1);
                var lb2 = Core.MovingAverageLookback(length2, maType2);
                return Math.Max(lb1, lb2) * 2 + 1 + deviation;
            }
        }
        public int TicksBack1
        {
            get
            {
                var lb1 = Core.MovingAverageLookback(length1, maType1);
                
                return lb1 * 2 + 1 + deviation;
            }
        }
        public int TicksBack2
        {
            get
            {
                var lb1 = Core.MovingAverageLookback(length2, maType2);

                return lb1 * 2 + 1 + deviation;
            }
        }
        #endregion

        public static Core.MAType ParseMAType(object value)
        {
            Core.MAType val;
            switch (value.ToString().ToLower())
            {
                case "sma":
                    val = Core.MAType.Sma;
                    break;
                case "ema":
                    val = Core.MAType.Ema;
                    break;
                case "dema":
                    val = Core.MAType.Dema;
                    break;
                case "tema":
                    val = Core.MAType.Tema;
                    break;
                case "trima":
                    val = Core.MAType.Trima;
                    break;
                case "t3":
                    val = Core.MAType.T3;
                    break;
                case "mama":
                    val = Core.MAType.Mama;
                    break;
                case "kama":
                    val = Core.MAType.Kama;
                    break;
                case "wma":
                    val = Core.MAType.Wma;
                    break;
                default:
                    val = Core.MAType.Sma;
                    break;
            }

            return val;
        }

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }

        double ParseDouble(object value, double lastValue)
        {
            var numStyle = System.Globalization.NumberStyles.Any;
            var cult = System.Globalization.CultureInfo.InvariantCulture;

            try
            {
                double output;
                double.TryParse(
                    value.ToString().Replace(',', '.'),
                    numStyle,
                    cult,
                    out output);
                return output;
            }
            catch
            {
                return lastValue;
            }
        }

        string ParseString(object value, string lastValue)
        {
            try
            {
                return Convert.ToString(value);
            }
            catch
            {
                return lastValue;
            }
        }
    }
}
